//
//  adc.c
//  TemplateProject
//
//  Created by Michael Köhler
//
//

#include "adc.h"
void adcInit(uint16_t prescaler, uint8_t refSource, uint8_t leftAlign){
    ADCSRA &= ~(1 << ADEN);
    ADMUX = 0;
    ADCSRA &= ~((1 << ADPS0)|(1 << ADPS1)|(1 << ADPS2));
    switch (prescaler) {
        case PSC2:
            ADCSRA |= (1 << ADPS0);
            break;
        case PSC4:
            ADCSRA |= (1 << ADPS1);
            break;
        case PSC8:
            ADCSRA |= (1 << ADPS0)|(1 << ADPS1);
            break;
        case PSC16:
            ADCSRA |= (1 << ADPS2);
            break;
        case PSC32:
            ADCSRA |= (1 << ADPS0)|(1 << ADPS2);
            break;
        case PSC64:
            ADCSRA |= (1 << ADPS1)|(1 << ADPS2);
            break;
        case PSC128:
            ADCSRA |= (1 << ADPS0)|(1 << ADPS1)|(1 << ADPS2);
            break;
        default:
            //kein richtiger Vorteiler uebergeben
            //vorteiler wird auf 0 gesetzt!
            break;
    }
    switch (refSource) {
        case AREF:
            //Referenzspannung vom Pin AREF
            break;
        case AVCC:
            //Referenzspannung von AVcc
            ADMUX |= (1 << REFS0);
            break;
        case INTERNAL:
            //Referenzspannung von der internen Referenz, 1.1V bei Atmega328p
            ADMUX |= (1 << REFS0)|(1 << REFS1);
            break;
        default:
            //keine richtige Referenzquelle gewaehlt
            //Referenzquelle wird vom Pin AREF erwartet
            break;
    }
    (leftAlign = LEFT) ? (ADMUX |= (1 << ADLAR)) : (ADMUX &= ~(1 << ADLAR));
    // adc starten (default: free running)
    ADCSRA |= (1 << ADEN)|(1 << ADSC);
    // und auf Abschluss der Konvertierung warten, Dummy-Read
    while (ADCSRA & (1 << ADSC)) {
        
    }
}

uint16_t adcRead(uint8_t channel){
    ADMUX = (ADMUX & ~(0x1f)) | (channel & 0x1f);
    if (ADCSRA & (1 << ADEN)) {
        //ADC ist eingeschaltet, Messung wird gestartet
        ADCSRA |= (1 << ADSC);
        while(ADCSRA & (1 << ADSC)){
            
        }
        if(ADMUX & (1 << ADLAR)){
            return (uint16_t)ADCH<<8;
        } else{
            return ADC;
        };
    }else{
        //error, ADC nicht eingeschaltet
        return 0xffff;
    }
}

uint16_t adcReadMittel(uint8_t channel, uint16_t cycles){
    uint16_t cycleCount;
    uint32_t result = 0;
    if (ADCSRA & (1 << ADEN)){
        for (cycleCount = 0; cycleCount < cycles; cycleCount++) {
            result += adcRead(channel);
        }
        return result/cycles;
    }else {
        //error, ADC nicht eingeschaltet
        return 0xffff;
    }
}
//Interrupt-Vector des ADC
ISR(ADC_vect){
    
}
