//
//  uart.c
//
//  Created by Michael Köhler
//

#include "uart.h"

volatile int8_t uart_str_count = 0;
volatile int8_t uart_str_complete = 0;
char uart_string[UART_MAXSTRLEN] = "";

void uartSetup(uint32_t baudRate, uint8_t stoppbits, uint8_t parity, uint8_t bitmode){
    // set Baudrate
    uint16_t ubbrValue = (uint16_t)((F_CPU+baudRate*8)/(baudRate*16)-1);
    UBRR0H = ubbrValue >> 8;
    UBRR0L = ubbrValue & 0xFF;
    // set Stopbit
    switch (stoppbits) {
        case ONE:
            UCSR0C = 0;
            break;
        case TWO:
            UCSR0C = (1 << USBS0);
            break;
            
        default:
            break;
    }
    // set Parity
    switch (parity) {
        case EVEN:
            UCSR0C |= (1 << UPM01);
            break;
        case ODD:
            UCSR0C |= (1 << UPM01)|(1 << UPM00);
            break;
        case NONE:
            UCSR0C &= ~((1 << UPM01)|(1 << UPM00));
            break;
        default:
            break;
    }
    // set Character size
    switch (bitmode) {
        case FIVE:
            UCSR0B &= ~(1 << UCSZ02);
            break;
        case SIX:
            UCSR0B &= ~(1 << UCSZ02);
            UCSR0C |= (1 << UCSZ00);
            break;
        case SEVEN:
            UCSR0B &= ~(1 << UCSZ02);
            UCSR0C |= (1 << UCSZ01);
            break;
        case EIGHT:
            UCSR0B &= ~(1 << UCSZ02);
            UCSR0C |= (1 << UCSZ01)|(1 << UCSZ00);
            break;
        case NINE:
            UCSR0B |= (1 << UCSZ02);
            UCSR0C |= (1 << UCSZ01)|(1 << UCSZ00);
            break;
        default:
            break;
    }
    // set TX/RX enable, RX-Interrupt enable
    UCSR0B |= (1<<TXEN0)|(1 << RXEN0)|(1<<RXCIE0);  // UART TX/RX einschalten, RXIE einschalten
}
void sendChar(unsigned char zeichen){
    {
        /* Wait for empty transmit buffer */
        while ( !( UCSR0A & (1<<UDRE0)) );
        /* Put data into buffer, sends the data */
        UDR0 = zeichen;
    }
}
void sendString(const char* charOfString){
    //send string from ram
	/* default: */
    /* while *charOfString != '\0 */
    while (*charOfString != stringComplete) {
        sendChar(*charOfString);
        charOfString++;
    }
}
void sendString_p(const char* charOfString){
    //send string from flash
    register char c;
	/* default: */
    /* while *charOfString != '\0 */
    while ( (c = pgm_read_byte(charOfString++)) != stringComplete) {
        sendChar(c);
    }
}

void sendString_e(const char* charOfString){
    //send string from eeprom
    register char c;
	/* default: */
    /* while *charOfString != '\0 */
    while ( (c = eeprom_read_byte((uint8_t*)charOfString++)) != stringComplete ) {
        sendChar(c);
    }
}
// Interrupts
// Uart-RX
ISR (USART_RX_vect) {
    unsigned char nextChar;
    // Daten aus dem Puffer lesen
    nextChar = UDR0;
    if( !(uart_str_complete > 0) ) {
        // wenn uart_string gerade in Verwendung, wird das neue Zeichen verworfen
        // ansonsten wird es hier ausgewertet.
        if( (nextChar != commandComplete) &&
           (uart_str_count < UART_MAXSTRLEN) ) {
            uart_string[uart_str_count] = nextChar;
            // carrige return (ASCII-Wert 13) wird verworfen!!
            if (nextChar != '\r') uart_str_count++;
        } else {
            uart_string[uart_str_count] = '\0';
            uart_str_count = 0;
            uart_str_complete = 1;
        }
    }
}
