//
//  uart.h
//
//  Created by Michael Köhler
//
//

#ifndef uart_h
#define uart_h

#include <avr/io.h>
#include <avr/interrupt.h>
#include <avr/pgmspace.h>
#include <avr/eeprom.h>

#define stringComplete			'\0'				// default: String-Ende-Zeichen Null-Zeichen (\0)
#define commandComplete			'\n'				// default: Befehls-Ende-Zeichen Line-Feed (\n)
#define defaultBaudrate         1200UL				// default: Baudrate 1200 Baud

#define UART_MAXSTRLEN          10					// max Stringlaenge (benoetigter RAM: Stringlaenge * sizeof(char))

#define UBRR_VAL (F_CPU+defaultBaudrate*8)/(defaultBaudrate*16)-1	// clever runden
#define BAUD_REAL (F_CPU/(16*(UBRR_VAL+1)))			                // Reale Baudrate
#define BAUD_ERROR ((BAUD_REAL*1000)/defaultBaudrate)		        // Fehler in Promille, 1000 = kein Fehler.

#if ((BAUD_ERROR<990) || (BAUD_ERROR>1010))
#error Systematischer Fehler der Baudrate grösser 1% und damit zu hoch!
#endif
enum UART_Parity{
    ODD,
    EVEN,
    NONE
};
enum UART_Stoppbits{
    ONE,
    TWO
};
enum UART_Bitmode{
    FIVE,
    SIX,
    SEVEN,
    EIGHT,
    NINE
};
volatile extern int8_t uart_str_count;
volatile extern int8_t uart_str_complete;
extern char uart_string[UART_MAXSTRLEN];

void uartSetup(uint32_t baudRate, uint8_t stoppbits, uint8_t parity, uint8_t bitmode);
void sendChar(unsigned char zeichen);
void sendString(const char* charOfString);
void sendString_p(const char* charOfString);
void sendString_e(const char* charOfString);

#endif /* uart_h */
